<?php
namespace Xoogu\W3TC;

class DynamicShortcodes_Admin extends DynamicShortcodes
{

    public function setBaseDir($sBaseDir)
    {
        $this->_sBaseDir = $sBaseDir;
    }

    public function __construct($sBaseDir, $ModifyWPConfig)
    {
        if (!parent::__construct()) {
            return;
        }

        $this->_sBaseDir = $sBaseDir;
        $this->_ModifyWPConfig = $ModifyWPConfig;

        //Load the internationalization
        load_plugin_textdomain('xoogu-dynamic-shortcodes-w3tc', false, $this->_sBaseDir.'/lang/');

        //If extension has been activated then page cache or late init disabled, deactivate extension
        if (!$this->_oW3TCConfig->get_boolean('pgcache.late_init') || !$this->_oW3TCConfig->get_boolean('pgcache.enabled')) {
            \W3TC\Extensions_Util::deactivate_extension('xoogu-dynamic-shortcodes-w3tc', $this->_oW3TCConfig);
            return;
        }

        //Displaying the admin settings page
        add_action( 'w3tc_extension_page_xoogu-dynamic-shortcodes-w3tc', array($this, 'printSettingsPage') );
    
        //Saving settings from admin page
        add_action( 'w3tc_config_ui_save-w3tc_extensions', array($this, 'processPOST'), 10, 2 );

        //Add activation and deactivation handlers to add / remove the W3TC_DYNAMIC_SECURITY constant to wp-config.php
        add_action('w3tc_deactivate_extension_xoogu-dynamic-shortcodes-w3tc', array($this, 'deactivate'));
        add_action('w3tc_activate_extension_xoogu-dynamic-shortcodes-w3tc', array($this, 'activate'));
        //Currently W3TC does not implement an extension activation hook, so have to look at URL to determine if extension has just been activated, then run activation based on that
        if (!empty($_GET['activated'])) {
            //have to check the url rather than using GET as W3TC includes multiple activated GET params (not indexed) when you bulk activate extensions
            if (strpos($_SERVER['REQUEST_URI'], 'activated=xoogu-dynamic-shortcodes-w3tc')) {
                do_action('w3tc_activate_extension_xoogu-dynamic-shortcodes-w3tc');
            }
        }
    }
    
    
    /**
     *Removes shortcode setting from settings when the form is saved with no shortcodes selected
     *(By default W3TC keeps the settings as they are when this happens, so this is a workaround to fix this problem)
     *@param W3_Config $oConfig The W3TC config settings object
     *@param W3_Config $oConfig The previous W3TC config settings object (prior to new settings being saved)
     *@return W3_Config The updated W3TC config settings object
     */
    public function processPOST($oConfig, $oOldConfig)
    {
        //check we're saving settings for our extension
        if ($_GET['extension'] != 'xoogu-dynamic-shortcodes-w3tc') {
            return;
        }

        //get the existing settings for all extensions
        $aExtensionsData = $oConfig->get_array('extensions.settings', array());
        //bool to hold whether any extra shortcodes were entered (and added)
        $bShortcodeAdded = false;

        //if no 'standard' shortcodes selected, we need to set our shortcodes to an empty array, since W3TC keeps the settings otherwise
        if (empty($_POST['extensions_settings_xoogu-dynamic-shortcodes-w3tc_shortcodes'])) {
            $this->aShortcodes = array();
        } else {
            $this->aShortcodes = $_POST['extensions_settings_xoogu-dynamic-shortcodes-w3tc_shortcodes'];
        }
        
        //parse any extra shortcodes
        if (!empty($_POST['extraShortcodes'])) {
            $aExtraShortcodes = explode(',', $_POST['extraShortcodes']);
            foreach ($aExtraShortcodes as $sExtraShortcode) {
                $sExtraShortcode = trim($sExtraShortcode);
                if ($sExtraShortcode && !in_array($sExtraShortcode, $this->aShortcodes)) {
                    $this->aShortcodes[] = $sExtraShortcode;
                }
            }
        }

        //save the settings
        $aExtensionsData['xoogu-dynamic-shortcodes-w3tc']['shortcodes'] = $this->aShortcodes;
        $oConfig->set('extensions.settings', $aExtensionsData);
        return $oConfig;
    }
    
    
    /**
     *Prints the list of shortcodes with checkboxes for making dynamic
     */
    public function printSettingsPage()
    {
        
        \W3TC\Util_Ui::postbox_header( 'Xoogu Dynamic Shortcodes' );

        echo '<p class="description">'.__('Allows selected shortcodes to be generated dynamically on every page load','xoogu-dynamic-shortcodes-w3tc').'</p>';
        echo '<h3>'.__('Shortcodes to make dynamic','xoogu-dynamic-shortcodes-w3tc').'</h3>';
        
        $sNameBase = 'extensions_settings_xoogu-dynamic-shortcodes-w3tc_shortcodes';
        $sIdBase = 'extensions_settings_xoogu-dynamic-shortcodes-w3tc_shortcodes';
        $aShortcodes = array_flip(array_merge(array_flip(array_keys($GLOBALS['shortcode_tags'])), array_flip($this->aShortcodes)));
        sort($aShortcodes);
        echo '<ul style="list-style-type: none">';
        foreach ($aShortcodes as $sShortCode) {
            $sId = $sIdBase.'_'.$sShortCode;
            $sName = $sNameBase.'[]';
            echo '<li><input type="checkbox" id="'.$sId.'" name="'.$sName.'" value="'.$sShortCode.'"';
            if (!empty($this->aShortcodes) && in_array($sShortCode, $this->aShortcodes)) {
                echo ' checked="checked"';
            }
            echo ' /><label for="'.$sId.'">'.$sShortCode.'</label></li>';
        }
        echo '</ul>';
        echo '<label for="extraShortcodes" >'.__('Any shortcodes not listed above you want to add (comma separated)','xoogu-dynamic-shortcodes-w3tc').'</label>';
        echo '<input type="text" value="" id="extraShortcodes" name="extraShortcodes" />';

        \W3TC\Util_Ui::button_config_save('extension_xoogu-dynamic-shortcodes-w3tc');
        
        \W3TC\Util_Ui::postbox_footer();
    }
    
    
    /**
     * Display if caching or not.
     */
    function extension_header()
    {
        echo '<p>';
        printf(__('The Dynamic Shortcodes extension is currently %s', 'xoogu-dynamic-shortcodes-w3tc'),
            '<span class="w3tc-enabled">' . __('enabled', 'w3-total-cache') . '</span>');
        echo '.</p>';
    }
    
    
    /**
     *Add W3TC_DYNAMIC_SECURITY constant to wp-config upon activation
     */
    public function activate()
    {
        //Add the W3TC_DYNAMIC_SECURITY constant
        if (!defined('W3TC_DYNAMIC_SECURITY')) {
            $this->_ModifyWPConfig->wp_config_add_directive($this->wp_config_addon());
        }
        
        //Ensure that HTML minification won't strip out the mfunc comments
        $aHTMLCommentsIgnoreList = $this->_oW3TCConfig->get_array('minify.html.comments.ignore', array());
        if (!in_array('mfunc', $aHTMLCommentsIgnoreList)) {
            $aHTMLCommentsIgnoreList[] = 'mfunc';
            $this->_oW3TCConfig->set('minify.html.comments.ignore', $aHTMLCommentsIgnoreList);
            $this->_oW3TCConfig->save();
        }
        
    }

    
    /**
     *Remove W3TC_DYNAMIC_SECURITY constant from wp-config upon deactivation
     *And remove mfunc from list of HTML minify comment exceptions
     */
    public function deactivate() {
        //remove mfunc from the list of minify comment exceptions
        $aHTMLCommentsIgnoreList = $this->_oW3TCConfig->get_array('minify.html.comments.ignore', array());
        $aHTMLCommentsIgnoreList = array_diff($aHTMLCommentsIgnoreList, array('mfunc'));
        $this->_oW3TCConfig->set('minify.html.comments.ignore', $aHTMLCommentsIgnoreList);
        $this->_oW3TCConfig->save();
        
        //Remove W3TC_DYNAMIC_SECURITY constant
        if (defined('W3TC_DYNAMIC_SECURITY')) {
            $this->_ModifyWPConfig->wp_config_remove_directive("define('W3TC_DYNAMIC_SECURITY', {W3TC_DYNAMIC_SECURITY});", "/\\s*define\\(\'W3TC_DYNAMIC_SECURITY\'.*\\r?\\n?/m");
        }
    }
    
    
    /**
     * @return string Line to add to wp-config.php to make dynamic code work in W3TC
     **/
    private function wp_config_addon() {
        return "define('W3TC_DYNAMIC_SECURITY', '".md5(mt_rand())."'); // ".__('Added by Xoogu Dynamic Shortcodes for W3 Total Cache', 'xoogu-dynamic-shortcodes-w3tc')."\r\n";
    }
}
