<?php
namespace Xoogu\W3TC;

/**
 * Allows adding / removing directives to wp-config.php
 * Based on w3-total-cache/PgCache_Environment.php
 */

class ModifyWPConfig
{
	/*
	 * wp-config modification
	 */

	/**
	 * Adds a given directive from wp-config.php
	 *
	 * @param string Directive to add
	 * @throws Util_WpFile_FilesystemOperationException with S/FTP form if it can't get the required filesystem credentials
	 */
	public function wp_config_add_directive($sDirective) {
		$config_path = \W3TC\Util_Environment::wp_config_path();

		$config_data = @file_get_contents( $config_path );
		if ( $config_data === false ) {
			exit( sprintf(__('Couldn\'t locate wp_config.php. Please add the following line to your WordPress configuation file: %s', 'xoogu-dynamic-shortcodes-w3tc'), $sDirective) );
		}

		$new_config_data = preg_replace('~<\?(php)?~', "\\0\n$sDirective", $config_data, 1);

		if ( $new_config_data != $config_data ) {
			try {
				\W3TC\Util_WpFile::write_to_file( $config_path, $new_config_data );
			} catch ( \W3TC\Util_WpFile_FilesystemOperationException $ex ) {
				throw new \W3TC\Util_WpFile_FilesystemModifyException(
					$ex->getMessage(), $ex->credentials_form(),
					'Edit file <strong>' . $config_path .
					'</strong> and add next lines:', $config_path,
					$sDirective );
			}
		}
	}


	/**
	 * Removes a given directive from wp-config.php
	 *
	 * @param $sDirective string Directive to remove
	 * @param $sDirectiveRegex string Regular Expression of directive to remove (optional, defaults to escaped version of $sDirective)
	 * @throws Util_WpFile_FilesystemOperationException with S/FTP form if it can't get the required filesystem credentials
	 */
	public function wp_config_remove_directive($sDirective, $sDirectiveRegex='') {
		$config_path = \W3TC\Util_Environment::wp_config_path();

		$config_data = @file_get_contents( $config_path );
		if ( $config_data === false ) {
			exit( sprintf(__('Couldn\'t locate wp_config.php. Please remove the following line from your WordPress configuation file: %s', 'xoogu-dynamic-shortcodes-w3tc'), $sDirective) );
		}
		if (!$sDirectiveRegex) {
			$sDirectiveRegex = preg_quote($sDirective);
		}
		$new_config_data = preg_replace($sDirectiveRegex, '', $config_data, 1);
		if ( $new_config_data != $config_data ) {
			try {
				\W3TC\Util_WpFile::write_to_file( $config_path, $new_config_data );
			} catch ( \W3TC\Util_WpFile_FilesystemOperationException $ex ) {
				throw new \W3TC\Util_WpFile_FilesystemModifyException(
					$ex->getMessage(), $ex->credentials_form(),
					'Edit file <strong>' . $config_path .
					'</strong> and remove next lines:',
					$config_path,  $sDirective );
			}
		}
	}

}
